@@ -1,6 +1,6 @@
 /*
  * sar and sadf common routines.
- * (C) 1999-2005 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1999-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -29,7 +29,6 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
-#include <sys/param.h>	/* for HZ */
 
 #include "sa.h"
 #include "common.h"
@@ -85,14 +84,8 @@ void salloc_cpu_array(struct stats_one_cpu *st_cpu[], unsigned int nr_cpu)
 {
    int i;
 
-   for (i = 0; i < DIM; i++) {
-      if ((st_cpu[i] = (struct stats_one_cpu *) malloc(STATS_ONE_CPU_SIZE * nr_cpu)) == NULL) {
-	 perror("malloc");
-	 exit(4);
-      }
-
-      memset(st_cpu[i], 0, STATS_ONE_CPU_SIZE * nr_cpu);
-   }
+   for (i = 0; i < DIM; i++)
+      SREALLOC(st_cpu[i], struct stats_one_cpu, STATS_ONE_CPU_SIZE * nr_cpu);
 }
 
 
@@ -105,14 +98,8 @@ void salloc_serial_array(struct stats_serial *st_serial[], int nr_serial)
 {
    int i;
 
-   for (i = 0; i < DIM; i++) {
-      if ((st_serial[i] = (struct stats_serial *) malloc(STATS_SERIAL_SIZE * nr_serial)) == NULL) {
-	 perror("malloc");
-	 exit(4);
-      }
-
-      memset(st_serial[i], 0, STATS_SERIAL_SIZE * nr_serial);
-   }
+   for (i = 0; i < DIM; i++)
+      SREALLOC(st_serial[i], struct stats_serial, STATS_SERIAL_SIZE * nr_serial);
 }
 
 
@@ -126,14 +113,9 @@ void salloc_irqcpu_array(struct stats_irq_cpu *st_irq_cpu[],
 {
    int i;
 
-   for (i = 0; i < DIM; i++) {
-      if ((st_irq_cpu[i] = (struct stats_irq_cpu *) malloc(STATS_IRQ_CPU_SIZE * nr_cpus * nr_irqcpu)) == NULL) {
-	 perror("malloc");
-	 exit(4);
-      }
-
-      memset(st_irq_cpu[i], 0, STATS_IRQ_CPU_SIZE * nr_cpus * nr_irqcpu);
-   }
+   for (i = 0; i < DIM; i++)
+      SREALLOC(st_irq_cpu[i], struct stats_irq_cpu,
+	       STATS_IRQ_CPU_SIZE * nr_cpus * nr_irqcpu);
 }
 
 
@@ -147,14 +129,8 @@ void salloc_net_dev_array(struct stats_net_dev *st_net_dev[],
 {
    int i;
 
-   for (i = 0; i < DIM; i++) {
-      if ((st_net_dev[i] = (struct stats_net_dev *) malloc(STATS_NET_DEV_SIZE * nr_iface)) == NULL) {
-	 perror("malloc");
-	 exit(4);
-      }
-
-      memset(st_net_dev[i], 0, STATS_NET_DEV_SIZE * nr_iface);
-   }
+   for (i = 0; i < DIM; i++)
+      SREALLOC(st_net_dev[i], struct stats_net_dev, STATS_NET_DEV_SIZE * nr_iface);
 }
 
 
@@ -167,14 +143,8 @@ void salloc_disk_array(struct disk_stats *st_disk[], int nr_disk)
 {
    int i;
 
-   for (i = 0; i < DIM; i++) {
-      if ((st_disk[i] = (struct disk_stats *) malloc(DISK_STATS_SIZE * nr_disk)) == NULL) {
-	 perror("malloc");
-	 exit(4);
-      }
-
-      memset(st_disk[i], 0, DISK_STATS_SIZE * nr_disk);
-   }
+   for (i = 0; i < DIM; i++)
+      SREALLOC(st_disk[i], struct disk_stats, DISK_STATS_SIZE * nr_disk);
 }
 
 
@@ -211,15 +181,18 @@ int next_slice(unsigned long long uptime_ref, unsigned long long uptime,
 {
    unsigned long file_interval, entry;
    static unsigned long long last_uptime = 0;
-   int min, max, pt1, pt2;
+   int min, max, pt1, pt2, cpu_nr;
    double f;
 
    if (!last_uptime || reset)
       last_uptime = uptime_ref;
 
+   cpu_nr = file_hdr->sa_proc;
+   if (!cpu_nr)
+      cpu_nr = 1;
+
    /* Interval cannot be greater than 0xffffffff here */
-   f = (((double) ((uptime - last_uptime) & 0xffffffff)) /
-	(file_hdr->sa_proc + 1)) / HZ;
+   f = (((double) ((uptime - last_uptime) & 0xffffffff)) / cpu_nr) / HZ;
    file_interval = (unsigned long) f;
    if ((f * 10) - (file_interval * 10) >= 5)
       file_interval++; /* Rounding to correct value */
@@ -239,8 +212,7 @@ int next_slice(unsigned long long uptime_ref, unsigned long long uptime,
     *       (Pn * Iu) or (P'n * Iu) belongs to In
     * with  Pn = En / Iu and P'n = En / Iu + 1
     */
-   f = (((double) ((uptime - uptime_ref) & 0xffffffff)) /
-	(file_hdr->sa_proc + 1)) / HZ;
+   f = (((double) ((uptime - uptime_ref) & 0xffffffff)) / cpu_nr) / HZ;
    entry = (unsigned long) f;
    if ((f * 10) - (entry * 10) >= 5)
       entry++;
@@ -282,16 +254,16 @@ int decode_timestamp(char timestamp[], struct tstamp *tse)
  * Compare two time stamps
  ***************************************************************************
  */
-int datecmp(struct tm *loc_time, struct tstamp *tse)
+int datecmp(struct tm *rectime, struct tstamp *tse)
 {
-   if (loc_time->tm_hour == tse->tm_hour) {
-      if (loc_time->tm_min == tse->tm_min)
-	return (loc_time->tm_sec - tse->tm_sec);
+   if (rectime->tm_hour == tse->tm_hour) {
+      if (rectime->tm_min == tse->tm_min)
+	return (rectime->tm_sec - tse->tm_sec);
       else
-	return (loc_time->tm_min - tse->tm_min);
+	return (rectime->tm_min - tse->tm_min);
    }
    else
-     return (loc_time->tm_hour - tse->tm_hour);
+     return (rectime->tm_hour - tse->tm_hour);
 }
 
 
@@ -314,6 +286,20 @@ int parse_timestamp(char *argv[], int *opt, struct tstamp *tse,
 }
 
 
+/*
+ ***************************************************************************
+ * Set current daily datafile
+ ***************************************************************************
+ */
+void set_default_file(struct tm *rectime, char *datafile)
+{
+   get_time(rectime);
+   snprintf(datafile, MAX_FILE_LEN,
+	    "%s/sa%02d", SA_DIR, rectime->tm_mday);
+   datafile[MAX_FILE_LEN - 1] = '\0';
+}
+
+
 /*
  ***************************************************************************
  * Set interval value.
@@ -341,7 +327,7 @@ void get_itv_value(struct file_stats *file_stats_curr,
    if (!(*g_itv))	/* Paranoia checking */
       *g_itv = 1;
 
-   if (nr_proc) {
+   if (nr_proc > 1) {
       if (!file_stats_prev->uptime0)
 	 *itv = file_stats_curr->uptime0;
       else
@@ -358,34 +344,48 @@ void get_itv_value(struct file_stats *file_stats_curr,
 
 /*
  ***************************************************************************
- * Print report header
+ * Fill rectime structure according to data saved in file header
  ***************************************************************************
  */
-void print_report_hdr(unsigned short format, unsigned int flags,
-		      struct tm *loc_time, struct file_hdr *file_hdr)
+void set_hdr_rectime(unsigned int flags, struct tm *rectime,
+		      struct file_hdr *file_hdr)
 {
+   struct tm *loc_t;
 
    if (PRINT_TRUE_TIME(flags)) {
       /* Get local time */
-      get_localtime(loc_time);
+      get_time(rectime);
 
-      loc_time->tm_mday = file_hdr->sa_day;
-      loc_time->tm_mon  = file_hdr->sa_month;
-      loc_time->tm_year = file_hdr->sa_year;
+      rectime->tm_mday = file_hdr->sa_day;
+      rectime->tm_mon  = file_hdr->sa_month;
+      rectime->tm_year = file_hdr->sa_year;
       /*
        * Call mktime() to set DST (Daylight Saving Time) flag.
        * Has anyone a better way to do it?
        */
-      loc_time->tm_hour = loc_time->tm_min = loc_time->tm_sec = 0;
-      mktime(loc_time);
+      rectime->tm_hour = rectime->tm_min = rectime->tm_sec = 0;
+      mktime(rectime);
    }
-   else
-      loc_time = localtime(&(file_hdr->sa_ust_time));
+   else {
+      loc_t = localtime(&(file_hdr->sa_ust_time));
+      *rectime = *loc_t;
+   }
+}
 
-   if (!format)
-      /* No output format (we are not using sadf) */
-      print_gal_header(loc_time, file_hdr->sa_sysname, file_hdr->sa_release,
-		       file_hdr->sa_nodename);
+
+/*
+ ***************************************************************************
+ * Print report header
+ ***************************************************************************
+ */
+void print_report_hdr(unsigned int flags, struct tm *rectime,
+		      struct file_hdr *file_hdr)
+{
+
+   set_hdr_rectime(flags, rectime, file_hdr);
+
+   print_gal_header(rectime, file_hdr->sa_sysname, file_hdr->sa_release,
+		    file_hdr->sa_nodename);
 }
 
 
@@ -571,10 +571,10 @@ unsigned long long get_per_cpu_interval(struct stats_one_cpu *st_cpu_i,
 {
    return ((st_cpu_i->per_cpu_user + st_cpu_i->per_cpu_nice +
 	    st_cpu_i->per_cpu_system + st_cpu_i->per_cpu_iowait +
-	    st_cpu_i->per_cpu_idle) -
+	    st_cpu_i->per_cpu_idle + st_cpu_i->per_cpu_steal) -
 	   (st_cpu_j->per_cpu_user + st_cpu_j->per_cpu_nice +
 	    st_cpu_j->per_cpu_system + st_cpu_j->per_cpu_iowait +
-	    st_cpu_j->per_cpu_idle));
+	    st_cpu_j->per_cpu_idle + st_cpu_j->per_cpu_steal));
 }
 
 
@@ -661,7 +661,7 @@ void prep_file_for_reading(int *ifd, char *dfile, struct file_hdr *file_hdr,
  ***************************************************************************
  */
 int parse_sar_opt(char *argv[], int opt, unsigned int *actflag,
-		  unsigned int *flags, short *dis_hdr, int caller,
+		  unsigned int *flags, int caller,
 		  unsigned char irq_bitmap[], unsigned char cpu_bitmap[])
 {
    int i;
@@ -683,34 +683,27 @@ int parse_sar_opt(char *argv[], int opt, unsigned int *actflag,
 	 break;
        case 'B':
 	 *actflag |= A_PAGE;
-	 (*dis_hdr)++;
 	 break;
        case 'b':
 	 *actflag |= A_IO;
-	 (*dis_hdr)++;
 	 break;
        case 'c':
 	 *actflag |= A_PROC;
-	 (*dis_hdr)++;
 	 break;
        case 'd':
 	 *actflag |= A_DISK;
-	 (*dis_hdr)++;
 	 break;
        case 'p':
 	 *flags |= S_F_DEV_PRETTY;
 	 break;
        case 'q':
 	 *actflag |= A_QUEUE;
-	 (*dis_hdr)++;
 	 break;
        case 'r':
 	 *actflag |= A_MEM_AMT;
-	 (*dis_hdr)++;
 	 break;
        case 'R':
 	 *actflag |= A_MEMORY;
-	 (*dis_hdr)++;
 	 break;
        case 't':
 	 if (caller == C_SAR)
@@ -720,25 +713,22 @@ int parse_sar_opt(char *argv[], int opt, unsigned int *actflag,
 	 break;
        case 'u':
 	 *actflag |= A_CPU;
-	 (*dis_hdr)++;
 	 break;
        case 'v':
 	 *actflag |= A_KTABLES;
-	 (*dis_hdr)++;
 	 break;
        case 'w':
 	 *actflag |= A_CTXSW;
-	 (*dis_hdr)++;
 	 break;
        case 'W':
 	 *actflag |= A_SWAP;
-	 (*dis_hdr)++;
 	 break;
        case 'y':
 	 *actflag |= A_SERIAL;
-	 (*dis_hdr)++;
 	 break;
        case 'V':
+	 print_version();
+	 break;
        default:
 	 return 1;
       }
@@ -752,8 +742,7 @@ int parse_sar_opt(char *argv[], int opt, unsigned int *actflag,
  * Parse sar "-n" option
  ***************************************************************************
  */
-int parse_sar_n_opt(char *argv[], int *opt, unsigned int *actflag,
-		    short *dis_hdr)
+int parse_sar_n_opt(char *argv[], int *opt, unsigned int *actflag)
 {
    if (!strcmp(argv[*opt], K_DEV))
       *actflag |= A_NET_DEV;
@@ -765,10 +754,8 @@ int parse_sar_n_opt(char *argv[], int *opt, unsigned int *actflag,
       *actflag |= A_NET_NFS;
    else if (!strcmp(argv[*opt], K_NFSD))
       *actflag |= A_NET_NFSD;
-   else if (!strcmp(argv[*opt], K_FULL)) {
+   else if (!strcmp(argv[*opt], K_ALL))
       *actflag |= A_NET_DEV + A_NET_EDEV + A_NET_SOCK + A_NET_NFS + A_NET_NFSD;
-      *dis_hdr = 9;
-   }
    else
       return 1;
 
@@ -783,7 +770,7 @@ int parse_sar_n_opt(char *argv[], int *opt, unsigned int *actflag,
  ***************************************************************************
  */
 int parse_sar_I_opt(char *argv[], int *opt, unsigned int *actflag,
-		    short *dis_hdr, unsigned char irq_bitmap[])
+		    unsigned char irq_bitmap[])
 {
    int i;
 
@@ -792,16 +779,13 @@ int parse_sar_I_opt(char *argv[], int *opt, unsigned int *actflag,
    else {
       *actflag |= A_ONE_IRQ;
       if (!strcmp(argv[*opt], K_ALL)) {
-	 *dis_hdr = 9;
 	 /* Set bit for the first 16 irq */
 	 irq_bitmap[0] = 0xff;
 	 irq_bitmap[1] = 0xff;
       }
-      else if (!strcmp(argv[*opt], K_XALL)) {
-	 *dis_hdr = 9;
+      else if (!strcmp(argv[*opt], K_XALL))
 	 /* Set every bit */
 	 init_bitmap(irq_bitmap, ~0, NR_IRQS);
-      }
       else {
 	 /*
 	  * Get irq number.
@@ -825,15 +809,13 @@ int parse_sar_I_opt(char *argv[], int *opt, unsigned int *actflag,
  ***************************************************************************
  */
 int parse_sa_P_opt(char *argv[], int *opt, unsigned int *flags,
-		   short *dis_hdr, unsigned char cpu_bitmap[])
+		   unsigned char cpu_bitmap[])
 {
    int i;
 
    if (argv[++(*opt)]) {
       *flags |= S_F_PER_PROC;
-      (*dis_hdr)++;
       if (!strcmp(argv[*opt], K_ALL)) {
-	 *dis_hdr = 9;
 	 /*
 	  * Set bit for every processor.
 	  * We still don't know if we are going to read stats
